@@ -2,7 +2,7 @@
 #
 #    Limited command Shell (lshell)
 #  
-#    $Id: lshell.py,v 1.65 2010/05/04 19:50:29 ghantoos Exp $
+#    $Id: lshell.py,v 1.80 2010/10/26 22:35:28 ghantoos Exp $
 #
 #    Copyright (C) 2008-2009 Ignace Mouzannar (ghantoos) <ghantoos@ghantoos.org>
 #
@@ -36,12 +36,13 @@ import re
 import getopt
 import logging
 import signal
+import subprocess
 import readline
 import grp
 import time
 
 __author__ = "Ignace Mouzannar (ghantoos) <ghantoos@ghantoos.org>"
-__version__ = "0.9.12"
+__version__ = "0.9.14"
 
 # Required config variable list per user
 required_config = ['allowed', 'forbidden', 'warning_counter'] 
@@ -108,7 +109,15 @@ class ShellCmd(cmd.Cmd, object):
         self.identchars = self.identchars + '+./-'
         self.log.error('Logged in')
         cmd.Cmd.__init__(self)
-        self.prompt = self.conf['username'] + ':~$ '
+        if self.conf.has_key('prompt'):
+            self.promptbase = self.conf['prompt']
+            self.promptbase = self.promptbase.replace('%u', getuser())
+            self.promptbase = self.promptbase.replace('%h', os.uname()[1])
+        else:
+            self.promptbase = getuser()
+
+        self.prompt = '%s:~$ ' % self.promptbase
+
         self.intro = self.conf['intro']
 
         # initialize cli variables
@@ -143,9 +152,18 @@ class ShellCmd(cmd.Cmd, object):
             self.log.info('CMD: "%s"' %self.g_line)
             if self.g_cmd == 'cd':
                 self.cd()
-            # list all allowed path
+            # builtin lpath function: list all allowed path
             elif self.g_cmd == 'lpath':
                 self.lpath()
+            # builtin lsudo function: list all allowed sudo commands
+            elif self.g_cmd == 'lsudo':
+                self.lsudo()
+            # builtin history function: print command history
+            elif self.g_cmd == 'history':
+                self.history()
+            # builtin export function
+            elif self.g_cmd == 'export':
+                self.export()
             else:
                 os.system(self.g_line)
         elif self.g_cmd not in ['', '?', 'help', None]: 
@@ -168,6 +186,41 @@ class ShellCmd(cmd.Cmd, object):
                 if path:
                     sys.stdout.write(" %s\n" %path[:-2])
 
+    def lsudo(self):
+        """ lists allowed sudo commands
+        """
+        if self.conf.has_key('sudo_commands'):
+            sys.stdout.write("Allowed sudo commands:\n")
+            for command in self.conf['sudo_commands']:
+                sys.stdout.write(" - %s\n" % command)
+
+    def history(self):
+        """ print the commands history
+        """
+        try:
+            try:
+                readline.write_history_file(self.conf['history_file'])
+            except IOError:
+                self.log.error('WARN: couldn\'t write history '                \
+                                   'to file %s\n' % self.conf['history_file'])
+            f = open(self.conf['history_file'], 'r')
+            i = 1
+            for item in f.readlines():
+                sys.stdout.write("%d:  %s" % (i, item) )
+                i += 1
+        except:
+            self.log.critical('** Unable to read the history file.')
+
+    def export(self):
+        """ export environment variables """
+        # if command contains at least 1 space
+        if self.g_line.count(' '):
+            env = self.g_line.split(" ", 1)[1]
+            # if it conatins the equal sign, consider only the first one
+            if env.count('='):
+                var, value = env.split(' ')[0].split('=')[0:2]
+                os.environ.update({var:value})
+
     def cd(self):
         """ implementation of the "cd" command
         """
@@ -269,23 +322,29 @@ class ShellCmd(cmd.Cmd, object):
     def counter_update(self, messagetype, path=None):
         """ Update the warning_counter, log and display a warning to the user
         """
-        self.conf['warning_counter'] -= 1
         if path:
             line = path
         else:
             line = self.g_line
-        if self.conf['warning_counter'] <= 0: 
+
+        # if warning_counter is set to -1, just warn, don't kick
+        if self.conf['warning_counter'] == -1:
             self.log.critical('*** forbidden %s -> "%s"'                       \
-                                                  % (messagetype ,line))
-            self.log.critical('*** Kicked out')
-            sys.exit(1)
+                                                      % (messagetype ,line))
         else:
-            self.log.critical('*** forbidden %s -> "%s"'                       \
-                                                  % (messagetype ,line))
-            self.stderr.write('*** You have %s warning(s) left,'               \
-                                ' before getting kicked out.\n'                \
-                                %(self.conf['warning_counter']-1))
-            self.stderr.write('This incident has been reported.\n')
+            self.conf['warning_counter'] -= 1
+            if self.conf['warning_counter'] < 0: 
+                self.log.critical('*** forbidden %s -> "%s"'                   \
+                                                      % (messagetype ,line))
+                self.log.critical('*** Kicked out')
+                sys.exit(1)
+            else:
+                self.log.critical('*** forbidden %s -> "%s"'                   \
+                                                      % (messagetype ,line))
+                self.stderr.write('*** You have %s warning(s) left,'           \
+                                    ' before getting kicked out.\n'            \
+                                    %(self.conf['warning_counter']))
+                self.stderr.write('This incident has been reported.\n')
 
     def check_path(self, line, completion=None, ssh=None):
         """ Check if a path is entered in the line. If so, it checks if user   \
@@ -310,9 +369,9 @@ class ShellCmd(cmd.Cmd, object):
             # replace "~" with home path
             item = os.path.expanduser(item)
             # if contains a shell variable
-            if re.findall('\$|\*|\?',item):
+            if re.findall('\$|\*|\?', item):
                 # remove quotes if available
-                item = re.sub("\"|\'","",item)
+                item = re.sub("\"|\'", "", item)
                 # expand shell variables (method 1)
                 #for var in re.findall(r'\$(\w+|\{[^}]*\})', item):
                 #    # get variable value (if defined)
@@ -323,7 +382,11 @@ class ShellCmd(cmd.Cmd, object):
                 #    item = re.sub('\$%s|\${%s}' %(var, var), value, item)
                 # expand shell variables and wildcards using "echo"
                 # i know, this a bit nasty...
-                cin, cout = os.popen2('`which echo` %s' % item)
+                p = subprocess.Popen( "`which echo` %s" % item,
+                                      shell=True,
+                                      stdin=subprocess.PIPE,
+                                      stdout=subprocess.PIPE )
+                (cin, cout) = (p.stdin, p.stdout)
                 item = cout.readlines()[0].split(' ')[0].strip()
                 item = os.path.expandvars(item)
             tomatch = os.path.realpath(item)
@@ -351,12 +414,12 @@ class ShellCmd(cmd.Cmd, object):
         """
 
         if path is self.conf['home_path']:
-            self.prompt = self.conf['username'] + ':~$ '
+            self.prompt = '%s:~$ ' % self.promptbase
         elif re.findall(self.conf['home_path'], path) :
-            self.prompt = self.conf['username'] + ':~'                         \
-                                + path.split(self.conf['home_path'])[1] + '$ '
+            self.prompt = '%s:~%s$ ' % ( self.promptbase, \
+                                         path.split(self.conf['home_path'])[1])
         else:
-            self.prompt = self.conf['username'] + ':' + path + '$ '
+            self.prompt = '%s:%s$ ' % (self.promptbase, path)
 
     def cmdloop(self, intro=None):
         """Repeatedly issue a prompt, accept input, parse an initial prefix    \
@@ -437,7 +500,8 @@ class ShellCmd(cmd.Cmd, object):
             endidx = readline.get_endidx() - stripped
             if line.split(' ')[0] == 'sudo' and len(line.split(' ')) <= 2:
                 compfunc = self.completesudo
-            elif line.split(' ')[0] in self.conf['allowed']:
+            elif len (line.split(' ')) > 1 \
+                 and line.split(' ')[0] in self.conf['allowed']:
                 compfunc = self.completechdir
             elif begidx > 0:
                 cmd, args, foo = self.parseline(line)
@@ -611,6 +675,7 @@ class CheckConfig:
         self.get_config()
         self.check_user_integrity()
         self.get_config_user()
+        self.check_env()
         self.check_scp_sftp()
         self.check_passwd()
 
@@ -671,6 +736,13 @@ class CheckConfig:
         sys.stderr.write('lshell-%s - Limited Shell\n' %__version__)
         sys.exit(0)
 
+    def check_env(self):
+        """ Load environment variable set in configuration file """
+        if self.conf.has_key('env_vars'):
+            env_vars = self.conf['env_vars']
+            for key in env_vars.keys():
+                os.environ[key] = str(env_vars[key])
+
     def check_file(self, file):
         """ This method checks the existence of the "argumently" given         \
         configuration file.
@@ -709,9 +781,19 @@ class CheckConfig:
                         4 : logging.DEBUG }
 
         # create logger for lshell application
-        logger = logging.getLogger('lshell')
-        formatter = logging.Formatter('%(asctime)s (' + getuser()              \
-                                                        + '): %(message)s')
+        if self.conf.has_key('syslogname'):
+            try:
+                logname = eval(self.conf['syslogname'])
+            except:
+                logfilename = self.conf['syslogname']
+        else:
+            logname = 'lshell'
+
+        logger = logging.getLogger(logname)
+        formatter = logging.Formatter('%%(asctime)s (%s): %%(message)s' \
+                                                % getuser() )
+        syslogformatter = logging.Formatter('%s[%s]: %s: %%(message)s' \
+                                                % (logname, os.getpid(), getuser() ))
 
         logger.setLevel(logging.DEBUG)
 
@@ -732,7 +814,10 @@ class CheckConfig:
 
         # read logfilename is exists, and set logfilename
         if self.conf.has_key('logfilename'):
-            logfilename = self.conf['logfilename']
+            try:
+                logfilename = eval(self.conf['logfilename'])
+            except:
+                logfilename = self.conf['logfilename']
             currentime = time.localtime()
             logfilename = logfilename.replace('%y','%s'   %currentime[0])
             logfilename = logfilename.replace('%m','%02d' %currentime[1])
@@ -745,13 +830,21 @@ class CheckConfig:
 
         if self.conf['loglevel'] > 0:
             try:
-                # if log file is writable add new log file handler
-                logfile = os.path.join(self.conf['logpath'], logfilename+'.log')
-                fp = open(logfile,'a').close()
-                self.logfile = logging.FileHandler(logfile)
-                logger.addHandler(self.logfile)
-                self.logfile.setFormatter(formatter)
-                self.logfile.setLevel(self.levels[self.conf['loglevel']])
+                if logfilename == "syslog":
+                    from logging.handlers import SysLogHandler
+                    syslog = SysLogHandler(address='/dev/log')
+                    syslog.setFormatter(syslogformatter)
+                    syslog.setLevel(self.levels[self.conf['loglevel']])
+                    logger.addHandler(syslog)
+                else:
+                    # if log file is writable add new log file handler
+                    logfile = os.path.join(self.conf['logpath'], \
+                                                            logfilename+'.log')
+                    fp = open(logfile,'a').close()
+                    self.logfile = logging.FileHandler(logfile)
+                    self.logfile.setFormatter(formatter)
+                    self.logfile.setLevel(self.levels[self.conf['loglevel']])
+                    logger.addHandler(self.logfile)
 
             except IOError:
                 # uncomment the 2 following lines to warn if log file is not   \
@@ -864,7 +957,11 @@ class CheckConfig:
     def expand_all(self):
         """ expand allowed, if set to 'all'
         """
-        expanded_all = []
+        # initialize list to common shell builtins
+        expanded_all = ['bg', 'break', 'case', 'cd', 'continue', 'eval', \
+                        'exec', 'exit', 'fg', 'if', 'jobs', 'kill', 'login', \
+                        'logout', 'set', 'shift', 'stop', 'suspend', 'umask', \
+                        'unset', 'wait', 'while' ]
         for directory in os.environ['PATH'].split(':'):
             if os.path.exists(directory):
                 for item in os.listdir(directory):
@@ -872,6 +969,7 @@ class CheckConfig:
                         expanded_all.append(item)
             else: self.log.error('CONF: PATH entry "%s" does not exist'        \
                                                                     % directory)
+
         return str(expanded_all)
  
     def myeval(self, value, info=''):
@@ -922,6 +1020,7 @@ class CheckConfig:
                     'forbidden',
                     'sudo_commands',
                     'warning_counter',
+                    'env_vars',
                     'timer',
                     'scp',
                     'scp_upload',
@@ -930,6 +1029,7 @@ class CheckConfig:
                     'overssh',
                     'strict',
                     'aliases',
+                    'prompt',
                     'history_size']:
             try:
                 self.conf[item] = self.myeval(self.conf_raw[item], item)
@@ -941,8 +1041,11 @@ class CheckConfig:
                 # default scp is allowed
                 elif item in ['scp_upload', 'scp_download']:
                     self.conf[item] = 1
-                elif item in ['aliases']:
+                elif item in ['aliases','env_vars']:
                     self.conf[item] = {}
+                # do not set the variable
+                elif item in ['prompt']:
+                    continue
                 else:
                     self.conf[item] = 0
             except TypeError:
@@ -1022,6 +1125,8 @@ class CheckConfig:
         # append default commands to allowed list
         self.conf['allowed'].append('exit')
         self.conf['allowed'].append('lpath')
+        self.conf['allowed'].append('lsudo')
+        self.conf['allowed'].append('history')
         self.conf['allowed'].append('clear')
 
         if self.conf['sudo_commands']:
@@ -1193,12 +1298,16 @@ def get_aliases(line, aliases):
     """ Replace all configured aliases in the line
     """
     for item in aliases.keys():
-        reg = '(^| |;)%s([ ;&\|]+|$)' % item
+        reg = '(^|; |;)%s([ ;&\|]+|$)' % item
         while re.findall(reg, line):
             beforecommand = re.findall(reg, line)[0][0]
             aftercommand = re.findall(reg, line)[0][1]
             line = re.sub(reg, "%s%s%s" % (beforecommand, aliases[item],       \
                                                      aftercommand), line, 1)
+            # if line does not change after sub, exit loop
+            linesave = line
+            if linesave == line:
+                break
     for char in [';', '&', '|']:
         # remove all remaining double char
         line = line.replace('%s%s' %(char, char), '%s' %char)
